#ifndef MYX_REDIS_BASE_INL_HPP_
#define MYX_REDIS_BASE_INL_HPP_

#pragma once

#ifndef MYXLIB_HEADER_ONLY
#include <myx/redis/base.hpp>
#endif

#include <myx/base/config.hpp>

#include <QTimer>

namespace myx {

namespace redis {

static const QByteArray Separator { "\r\n" }; //!< Строковый разделитель слов во фразе

/*!
 * \brief Создает объект класса
 * \param host Имя хоста сервера
 * \param port Номер порта сервера
 * \param parent Родительский объект
 */
Base::Base( QObject* parent ) :
	QObject{ parent }
{
	_readTimer  = new QTimer { this };
	_writeTimer = new QTimer { this };
	_readTimer->setSingleShot( true );
	_writeTimer->setSingleShot( true );
	_readTimer->setInterval( 100 );
	_writeTimer->setInterval( 100 );
	connect( _readTimer, &QTimer::timeout, [this]() {
		if ( _connectionFlags & Connected && _connectionFlags & Read )
		{
			_connectionFlags -= Read;
			emit connectionStateChanged( _connectionFlags );
		}
	} );
	connect( _writeTimer, &QTimer::timeout, [this]() {
		if ( _connectionFlags & Connected && _connectionFlags & Write )
		{
			_connectionFlags -= Write;
			emit connectionStateChanged( _connectionFlags );
		}
	} );
}


/*!
 * \brief Задает адрес хоста
 * \param host Адрес
 */
void Base::setHost( const QString& host )
{
	_host = host;
}


/*!
 * \brief Извлекает длину токена из буфера данных
 * \param pos Позиция в буфере
 * \param buffer Буфер данных
 */
int Base::fetchLength( QByteArray& buffer, int& pos )
{
	if ( ( buffer[pos] != '$' ) && ( buffer[pos] != '*' ) && ( buffer[pos] != ':' ) )
	{
		buffer.clear();
		pos = 0;
		return( -1 );
	}
	pos++;
	QByteArray text;
	while ( pos < buffer.length() )
	{
		if ( QChar( buffer[pos] ).isNumber() )
		{
			text += buffer[pos++];
		}
		else
		{
			pos += Separator.length();
			return( text.toInt() );
		}
	}
	return( -1 );
} // Base::fetchLength


/*!
 * \brief Добавляет к состоянию флаг Read
 */
void Base::onReadyRead()
{
	if ( _connectionFlags & Connected && !( _connectionFlags & Read ) )
	{
		_connectionFlags += Read;
		emit connectionStateChanged( _connectionFlags );
	}
	_readTimer->start();
}


/*!
 * \brief Добавляет к состоянию флаг Read
 */
void Base::onBytesWritten()
{
	if ( _connectionFlags & Connected && !( _connectionFlags & Write ) )
	{
		_connectionFlags += Write;
		emit connectionStateChanged( _connectionFlags );
	}
	_writeTimer->start();
}


/*!
 * \brief Выставляет флаг ошибки соединения
 */
void Base::onSocketError()
{
	emit connectionStateChanged( _connectionFlags = Error );
}

} // namespace redis

} // namespace myx

#endif // ifndef MYX_REDIS_BASE_INL_HPP_
