#ifndef MYX_REDIS_BASE_HPP_
#define MYX_REDIS_BASE_HPP_

#pragma once

#include <myx/base/config.hpp>

#include <QHash>
#include <QObject>

QT_FORWARD_DECLARE_CLASS( QTimer )
QT_FORWARD_DECLARE_CLASS( QTcpSocket )

namespace myx {

namespace redis {

/*!
 * \brief Базовый класс для объектов взаимодействия с redis
 */
class Base : public QObject
{
	Q_OBJECT

public:
	/*!
	 * \brief Состояние соединения
	 */
	enum ConnectionStateFlags
	{
		Unconnected = 1,  //!< Соединение не установлено
		Connecting  = 2,  //!< Соединение устанавливается
		Connected   = 4,  //!< Соединение установлено
		Read        = 8,  //!< Есть данные для чтения
		Write       = 16, //!< Есть данные для записи
		Error       = 32, //!< Ошибка соединения
	};

	Base( QObject* parent = nullptr );
	void setHost( const QString& host );
	/*!
	 * \brief Сигнализирует об изменении состояния соединения
	 * \param flags Флаги состояния
	 */
	Q_SIGNAL void connectionStateChanged( int flags );

protected:
	static constexpr int Timeout { 2000 };                 //!< Таймаут на сетевые операции
	int                  _connectionFlags { Unconnected }; //!< Флаги состояния соединения
	quint16              _port { 6379 };                   //!< Номер порта подключения к БД
	QString              _host { "127.0.0.1" };            //!< Имя хоста подключения к БД
	QTimer*              _readTimer;                       //!< Таймер чтения данных
	QTimer*              _writeTimer;                      //!< Таймер записи данных

	int fetchLength( QByteArray& buffer, int& pos );
	Q_SLOT void onReadyRead();
	Q_SLOT void onBytesWritten();
	Q_SLOT void onSocketError();
}; // class Base

} // namespace redis

} // namespace myx

#ifdef MYXLIB_HEADER_ONLY
#include "base-inl.hpp"
#endif

#endif // ifndef MYX_REDIS_BASE_HPP_
