#include <QVariant>

namespace myx {

namespace redis {

/*!
 * \brief Запаковывает пару значений в сообщение
 * \param first Первое значение
 * \param second Второе значение
 * \return Текстовое сообщение
 */
template< typename T >
QByteArray packPair( T first, T second )
{
	return( packList( { QVariant::fromValue( first ), QVariant::fromValue( second ) } ) );
}


/*!
 * \brief Запаковывает пару значений в сообщение
 * \param first Первое значение
 * \param second Второе значение
 * \return Текстовое сообщение
 */
template< typename T1, typename T2 >
QByteArray packPair( T1 first, T2 second )
{
	return( packList( { QVariant::fromValue( first ), QVariant::fromValue( second ) } ) );
}


/*!
 * \brief Распаковывает сообщение в пару однотипных значений
 * \param pair Текст сообщения
 * \return Пара значений
 */
template< typename T >
std::pair< T, T > unpackPair( const QByteArray& message )
{
	auto list = unpackList( message );
	if ( list.size() == 2 )
	{
		return { QVariant( list.first() ).value< T >(), QVariant( list.last() ).value< T >() };
	}
	return {};
}


/*!
 * \brief Распаковывает сообщение в пару однотипных значений
 * \param pair Текст сообщения
 * \return Пара значений
 */
template< typename T1, typename T2 >
std::pair< T1, T2 > unpackPair( const QByteArray& message )
{
	auto list = unpackList( message );
	if ( list.size() == 2 )
	{
		return { QVariant( list.first() ).value< T1 >(), QVariant( list.last() ).value< T2 >() };
	}
	return {};
}

} // namespace redis

} // namespace myx
